@@ -17,35 +17,35 @@
 extern crate clap;
 #[macro_use]
 extern crate lazy_static;
-use crate::task::Runner;
 use crate::chompfile::ChompTaskMaybeTemplated;
 use crate::chompfile::Chompfile;
+use crate::extensions::expand_template_tasks;
 use crate::extensions::init_js_platform;
 use crate::extensions::ExtensionEnvironment;
-use crate::task::expand_template_tasks;
+use crate::task::Runner;
 use anyhow::{anyhow, Result};
 use clap::{App, Arg};
 use std::collections::BTreeMap;
-use std::collections::HashMap;
 use std::collections::HashSet;
 use std::fs;
 extern crate num_cpus;
+use crate::engines::replace_env_vars_static;
 use hyper::Uri;
 use std::env;
 use std::fs::canonicalize;
-use crate::engines::replace_env_vars_static;
+use tokio::sync::mpsc::unbounded_channel;
 
 mod ansi_windows;
 mod chompfile;
 mod engines;
 mod extensions;
 mod http_client;
-mod serve;
 mod task;
+mod server;
 
 use std::path::PathBuf;
 
-const CHOMP_CORE: &str = "https://ga.jspm.io/npm:@chompbuild/extensions@0.1.14/";
+const CHOMP_CORE: &str = "https://ga.jspm.io/npm:@chompbuild/extensions@0.1.16/";
 
 const CHOMP_INIT: &str = r#"version = 0.1
 
@@ -70,8 +70,12 @@ fn uri_parse(uri_str: &str) -> Option<Uri> {
 
 #[tokio::main]
 async fn main() -> Result<()> {
+    #[cfg(not(debug_assertions))]
+    let version = "0.1.10";
+    #[cfg(debug_assertions)]
+    let version = "0.1.10-debug";
     let matches = App::new("Chomp")
-        .version("0.1.0")
+        .version(version)
         .arg(
             Arg::with_name("watch")
                 .short("w")
@@ -180,13 +184,14 @@ async fn main() -> Result<()> {
 
     #[cfg(target_os = "windows")]
     match ansi_windows::enable_ansi_support() {
-        Ok(()) => {},
+        Ok(()) => {}
         Err(_) => {
             // TODO: handling disabling of ansi codes
         }
     };
 
     let mut targets: Vec<String> = Vec::new();
+    let mut use_default_target = true;
     match matches.values_of("target") {
         Some(target) => {
             for item in target {
@@ -268,7 +273,10 @@ async fn main() -> Result<()> {
         global_env.insert(key.to_uppercase(), value);
     }
     for (key, value) in &chompfile.env {
-        global_env.insert(key.to_uppercase(), replace_env_vars_static(value, &global_env));
+        global_env.insert(
+            key.to_uppercase(),
+            replace_env_vars_static(value, &global_env),
+        );
     }
     if matches.is_present("eject_templates") {
         global_env.insert("CHOMP_EJECT".to_string(), "1".to_string());
@@ -276,7 +284,12 @@ async fn main() -> Result<()> {
     global_env.insert("CHOMP_POOL_SIZE".to_string(), pool_size.to_string());
     // extend global env with the chompfile env as well
     for (key, value) in &chompfile.env_default {
-        global_env.insert(key.to_uppercase(), replace_env_vars_static(value, &global_env));
+        if !global_env.contains_key(&key.to_uppercase()) {
+            global_env.insert(
+                key.to_uppercase(),
+                replace_env_vars_static(value, &global_env),
+            );
+        }
     }
 
     let mut extension_env = ExtensionEnvironment::new(&global_env);
@@ -355,6 +368,10 @@ async fn main() -> Result<()> {
     }
     extension_env.seal_extensions();
 
+    // channel for watch events
+    let (watch_event_sender, watch_event_receiver) = unbounded_channel();
+    // channel for adding new files to watcher
+    let (watch_sender, watch_receiver) = unbounded_channel();
     let mut serve_options = chompfile.server.clone();
     {
         if let Some(root) = matches.value_of("server-root") {
@@ -364,12 +381,8 @@ async fn main() -> Result<()> {
             serve_options.port = port.parse().unwrap();
         }
         if matches.is_present("serve") {
-            tokio::spawn(async move {
-                if let Err(e) = serve::serve(serve_options).await {
-                    eprintln!("{:?}", e);
-                    std::process::exit(1);
-                }
-            });
+            use_default_target = false;
+            tokio::spawn(server::serve(serve_options, watch_event_receiver, watch_sender));
         }
     }
 
@@ -380,40 +393,22 @@ async fn main() -> Result<()> {
         }
     }
 
+    let (mut has_templates, mut template_tasks) =
+        expand_template_tasks(&chompfile, &mut extension_env)?;
+    chompfile.task = Vec::new();
+    for task in extension_env.get_tasks().drain(..) {
+        has_templates = true;
+        chompfile.task.push(task.into());
+    }
+    chompfile.task.append(&mut template_tasks);
+
     if matches.is_present("format")
         || matches.is_present("eject_templates")
         || matches.is_present("list")
         || matches.is_present("import_scripts")
     {
+        use_default_target = false;
         if matches.is_present("eject_templates") {
-            let (mut has_templates, mut template_tasks) =
-                expand_template_tasks(&chompfile, &mut extension_env)?;
-            chompfile.task = Vec::new();
-            for task in extension_env.get_tasks().drain(..) {
-                has_templates = true;
-                chompfile.task.push(ChompTaskMaybeTemplated {
-                    target: task.target,
-                    targets: task.targets,
-                    args: task.args,
-                    cwd: task.cwd,
-                    dep: task.dep,
-                    deps: task.deps,
-                    display: task.display,
-                    engine: task.engine,
-                    env: task.env.unwrap_or_default(),
-                    env_default: task.env_default.unwrap_or_default(),
-                    env_replace: task.env_replace,
-                    invalidation: task.invalidation,
-                    validation: task.validation,
-                    run: task.run,
-                    name: task.name,
-                    serial: task.serial,
-                    stdio: task.stdio,
-                    template: task.template,
-                    template_options: task.template_options
-                });
-            }
-            chompfile.task.append(&mut template_tasks);
             if !has_templates {
                 return Err(anyhow!(
                     "\x1b[1m{}\x1b[0m has no templates to eject",
@@ -460,27 +455,10 @@ async fn main() -> Result<()> {
                         for (name, val) in scripts.iter() {
                             if let serde_json::Value::String(run) = &val {
                                 script_tasks = script_tasks + 1;
-                                chompfile.task.push(ChompTaskMaybeTemplated {
-                                    name: Some(name.to_string()),
-                                    run: Some(run.to_string()),
-                                    args: None,
-                                    cwd: None,
-                                    deps: None,
-                                    dep: None,
-                                    targets: None,
-                                    target: None,
-                                    display: None,
-                                    engine: None,
-                                    env_replace: None,
-                                    env: HashMap::new(),
-                                    env_default: HashMap::new(),
-                                    invalidation: None,
-                                    validation: None,
-                                    serial: None,
-                                    stdio: None,
-                                    template: None,
-                                    template_options: None,
-                                })
+                                let mut task = ChompTaskMaybeTemplated::new();
+                                task.name = Some(name.to_string());
+                                task.run = Some(run.to_string());
+                                chompfile.task.push(task);
                             }
                         }
                     }
@@ -522,16 +500,30 @@ async fn main() -> Result<()> {
         }
     }
 
-    let mut runner = Runner::new(&chompfile, &mut extension_env, pool_size, matches.is_present("serve") || matches.is_present("watch"))?;
-    let ok = runner.run(task::RunOptions {
-        watch: matches.is_present("serve") || matches.is_present("watch"),
-        force: matches.is_present("force"),
-        rerun: matches.is_present("rerun"),
-        args: if args.len() > 0 { Some(args) } else { None },
+    let targets = if targets.len() == 0 && use_default_target {
+        match &chompfile.default_task {
+            Some(default_task) => vec![default_task.clone()],
+            None => return Err(anyhow!("No default task provided. Set:\x1b[36m\n\n  default-task = '[taskname]'\n\n\x1b[0min the \x1b[1mchompfile.toml\x1b[0m to configure a default build task.")),
+        }
+    } else { targets };
+
+    let mut runner = Runner::new(
+        &chompfile,
+        &mut extension_env,
         pool_size,
-        targets,
-        cfg_file,
-    }).await?;
+        matches.is_present("serve") || matches.is_present("watch"),
+    )?;
+    let ok = runner
+        .run(task::RunOptions {
+            watch: matches.is_present("serve") || matches.is_present("watch"),
+            force: matches.is_present("force"),
+            rerun: matches.is_present("rerun"),
+            args: if args.len() > 0 { Some(args) } else { None },
+            pool_size,
+            targets,
+            cfg_file,
+        }, watch_event_sender, watch_receiver)
+        .await?;
 
     if !ok {
         eprintln!("Unable to complete all tasks.");
